@@ -181,6 +181,7 @@ public:
 		STATE_RING,
 		STATE_PICKUP,
 		STATE_SEIZE,
+		STATE_ANSWER,
 		STATE_STEP,
 		STATE_EXEC,
 		STATE_THREAD,
@@ -2144,6 +2145,7 @@ protected:
 	virtual bool enterRelease(Event *event);
 	virtual bool enterRinging(Event *event);
 	virtual bool enterPickup(Event *event);
+	virtual bool enterAnswer(Event *event);
 	virtual bool enterSeize(Event *event);
 	virtual bool enterHunting(Event *event);
 	virtual bool enterHangup(Event *event);
@@ -2179,6 +2181,7 @@ protected:
 	bool stateStandby(Event *event);
 	bool stateRinging(Event *event);
 	bool statePickup(Event *event);
+	bool stateAnswer(Event *event);
 	bool stateSeize(Event *event);
 	bool stateHunting(Event *event);
 	bool stateRunning(Event *event);
