@@ -429,12 +429,7 @@ void BayonneSysexec::allocate(const char *libexec, size_t bs, int pri, const cha
 	for(int i = 3; i < 100; ++i)
 		::close(i);
 
-	if(Bayonne::provision_test)
-		snprintf(path, sizeof(path), "%s/server/%s",
-			SOURCE_FILES, provision_libexec);
-	else
-		snprintf(path, sizeof(path), "%s/%s",
-			LIBDIR_FILES, provision_libexec);		
+	snprintf(path, sizeof(path), "%s/libexec", LIBDIR_FILES);		
 
 	execv(path, argv);
 	slog.error("libexec failed; exiting...");
