@@ -1607,7 +1607,7 @@ const char *BayonneSession::getAudio(bool live)
 
 	if(cp && *cp)
 	{
-		audio.encoding = Audio::getEncoding(cp); 
+		audio.encoding = Bayonne::getEncoding(cp); 
 
 		if(!ext)
 			ext = Audio::getExtension(audio.encoding);
