@@ -1903,6 +1903,32 @@ seize:
 	}
 }
 
+bool BayonneSession::stateAnswer(Event *event)
+{
+	if(enterAnswer(event))
+		return true;
+
+	switch(event->id)
+	{
+	case ENTER_STATE:
+		startTimer(50);
+		return true;
+	case TIMER_EXPIRED:
+		advance();
+		setRunning();
+		return true;
+	case LINE_DISCONNECT:
+		return true;
+	case AUDIO_DISCONNECT:
+	case AUDIO_RECONNECT:
+	case DTMF_KEYDOWN:
+	case DTMF_KEYUP:
+		return false;
+	default:
+		return enterCommon(event);
+	}
+}
+
 bool BayonneSession::statePickup(Event *event)
 {
 	if(event->id == ENTER_STATE && !offhook)
@@ -2792,6 +2818,11 @@ bool BayonneSession::enterHangup(Event *event)
 	return false;
 }
 
+bool BayonneSession::enterAnswer(Event *event)
+{
+	return false;
+}
+
 bool BayonneSession::enterPickup(Event *event)
 {
 	return false;
