@@ -166,6 +166,8 @@ Assoc()
 		InetAddress host(buf);
 		snprintf(buf, sizeof(buf), "%s:5060", 
 			inet_ntoa(host.getAddress()));
+		if(!stricmp(buf, "0.0.0.0:5060"))
+			setString(buf, sizeof(buf), "127.0.0.1:5060");
 		setValue("interface", buf);
 		cp = getLast("interface");
 	}
@@ -1348,12 +1350,14 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	if(ext || gw)
 	{
 		if(!ScriptChecks::useKeywords(line, 
-			"=reconnect=peering=secret=encoding=framing=dtmf=user=count=device=info=answer=accept"))
+			"=reconnect=peering=secret=encoding=framing=dtmf=user=count=device"
+			"=info=answer=accept=hold=transfer=update"))
 				return "invalid keywords used for sip external";
 	}
 	else if(!ScriptChecks::useKeywords(line, 
 		"=uri=timeout=server=proxy=userid=secret=type=realm=dtmf=reconnect"
-		"=limit=public=encoding=framing=local=caller=service=peering=answer=gateways=accept"))
+		"=limit=public=encoding=framing=local=caller=service=peering=answer=gateways"
+		"=accept=hold=transfer=update"))
 			return "invalid keywords for sip registry";
 
 	reg->protocol = "sip";
@@ -1371,6 +1375,9 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	reg->caller = ScriptChecks::findKeyword(line, "caller");
 	reg->reconnect = ScriptChecks::findKeyword(line, "reconnect");
 	reg->answer = ScriptChecks::findKeyword(line, "answer");
+	reg->update = ScriptChecks::findKeyword(line, "update");
+	reg->hold = ScriptChecks::findKeyword(line, "hold");
+	reg->transfer = ScriptChecks::findKeyword(line, "transfer");
 	gateways = ScriptChecks::findKeyword(line, "gateways");
 	
 	cp = ScriptChecks::findKeyword(line, "accept");
@@ -1381,6 +1388,7 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 		reg->localid = ScriptChecks::findKeyword(line, "user");
 	if(gw && !reg->localid)
 		reg->localid = ScriptChecks::findKeyword(line, "userid");
+
 	if(!ext)
 	{
 		reg->service = ScriptChecks::findKeyword(line, "service");
@@ -1469,6 +1477,15 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 			if(cp)
 				reg->framing = atol(cp);
 		}
+		cp = cfg->getLast("update");
+		if(cp && !reg->update)
+			reg->update = ip->dupString(cp);
+		cp = cfg->getLast("transfer");
+		if(cp && !reg->transfer)
+			reg->transfer = ip->dupString(cp);
+		cp = cfg->getLast("hold");
+		if(cp && !reg->hold)
+			reg->hold = ip->dupString(cp);
 		cp = cfg->getLast("accept");
 		if(cp && !reg->accept)
 			reg->accept = atoi(cp);
@@ -1486,6 +1503,20 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 		delete cfg;
 	}
 #endif
+	if(ext)
+	{
+		if(!reg->hold)
+			reg->hold = "send";
+		if(!reg->transfer)
+			reg->transfer = "send";
+	}
+	if(!reg->update)
+			reg->update = "invite";
+	if(reg->hold && !strnicmp(reg->hold, "no", 2))
+		reg->hold = NULL;
+	if(reg->transfer && !strnicmp(reg->transfer, "no", 2))
+		reg->transfer = NULL;
+
 	if(reg->peering)
 	{
 		if(!stricmp(reg->peering, "yes"))
@@ -2368,6 +2399,16 @@ killit:
 				registerRequest(sevent);
 			else if(MSG_IS_OPTIONS(sevent->request))
 				requestOptions(sevent);
+			else if(MSG_IS_UPDATE(sevent->request))
+				goto answer;
+			else if(MSG_IS_PRACK(sevent->request))
+			{
+				if(eXosip_get_sdp_info(sevent->request))
+					goto answer;
+				eXosip_lock();
+				eXosip_call_send_answer(sevent->tid, 200, NULL);
+				eXosip_unlock();
+			}
 			else if(MSG_IS_PUBLISH(sevent->request))
 			{
 				eXosip_call_get_referto(sevent->did, buf, sizeof(buf));
@@ -2408,6 +2449,7 @@ killit:
 			else
 				slog.debug("sip: out of dialog msg %s", sevent->request->sip_method);
 			break;
+answer:
 		case EXOSIP_CALL_REINVITE:
                         if(!sevent->request)
                                 break;
@@ -2565,6 +2607,13 @@ killit:
 					data_rtpmap);
 				addString(sdp, 4096, cbuf);
 			}
+			if(session->info.framing)
+			{
+				snprintf(cbuf, sizeof(cbuf),
+					"a=ptime:%ld\r\n",
+					session->info.framing);
+				addString(sdp, 4096, cbuf);
+			}		
 			memset(&event, 0, sizeof(event));
 			if(*conn->c_addr == '0' || inactive)
 				event.id = CALL_HOLD;
@@ -2630,6 +2679,7 @@ killit:
 			if(sevent->cid < 1 && sevent->did < 1)
 				break;
 
+			inactive = false;
 			remote_media = NULL;
 			local_media = NULL;
 			conn = NULL;
@@ -3008,12 +3058,16 @@ sip415:
 				audio_payload = NULL;
 				tmp = NULL;
 				pos = 0;
-				
+				inactive = false;				
+
 				while(!osip_list_eol(remote_media->a_attributes, pos))
 				{
 					attr = (sdp_attribute_t *)osip_list_get(remote_media->a_attributes, pos++);
 					if(!attr)
 						continue;
+					tmp = attr->a_att_field;
+					if(tmp && !stricmp(tmp, "inactive"))
+						inactive = true;
 					tmp = attr->a_att_value;
 					if(!tmp)
 						continue;
@@ -3125,6 +3179,13 @@ sip415:
 					session->data_payload, data_rtpmap);
 				addString(sdp, 4096, cbuf);
 			}
+			if(session->info.framing)
+			{
+				snprintf(cbuf, sizeof(cbuf),
+					"a=ptime:%ld\r\n",
+					session->info.framing);
+				addString(sdp, 4096, cbuf);
+			}
 
                         if(!s->postEvent(&event))
                                 goto sip404;
@@ -3133,19 +3194,77 @@ sip415:
 			cp = NULL;
 			if(reg)
 				cp = reg->answer;
-			if(!cp)
+			if(!cp && inactive)
+				cp = "183";
+			else if(!cp)
 				cp = "200";
 
-			if(strstr(cp, "180") || !stricmp(cp, "no"))
+			if(!stricmp(cp, "no"))
+				cp = "183";
+
+			if(!stricmp(cp, "yes") || !stricmp(cp, "fast"))
+				cp = "200";
+
+			if(!stricmp(cp, "100"))
+			{
+				eXosip_lock();
+				eXosip_call_send_answer(sevent->tid, 100, NULL);
+				eXosip_unlock();
+				cp = "200";
+			}
+
+			if(!stricmp(cp, "ims") || !strnicmp(cp, "3g", 2))
+				cp = "183";
+
+			if(!strnicmp(cp, "ring", 4))
+			{
+				eXosip_lock();
+				eXosip_call_send_answer(sevent->tid, 100, NULL);
+				eXosip_unlock();
+				cp = "180";
+			}
+			// maybe we have "100,xxx" as in "100,182"
+			else if(strstr(cp, "100"))
+			{
+				eXosip_lock();
+				eXosip_call_send_answer(sevent->tid, 100, NULL);
+				eXosip_unlock();
+			}
+			if(strstr(cp, "183") && strstr(cp, "200"))
+			{
+				eXosip_lock();
+				eXosip_call_send_answer(sevent->tid, 183, NULL);
+				eXosip_unlock();
+				cp = "200";
+			}
+			if(strstr(cp, "180"))
 			{
 				eXosip_lock();
 				eXosip_call_send_answer(sevent->tid, 180, NULL);	
 				eXosip_unlock();
 				Thread::yield();
+				if(!strstr(cp, "200"))
+				{
+					session->tid = sevent->tid;
+					session->sip_answered = false;
+					session = NULL;
+					goto clear;
+				}
 			}
 
 			eXosip_lock();
-			eXosip_call_build_answer(sevent->tid, 200, &msg);
+			if(strstr(cp, "183") || strstr(cp, "182"))
+			{
+				session->tid = sevent->tid;
+				session->sip_answered = false;
+				eXosip_call_build_answer(sevent->tid, atoi(cp), &msg);
+			}
+			else
+			{
+				session->sip_answered = true;				
+				eXosip_call_build_answer(sevent->tid, 200, &msg);
+			}
+
 			if(!msg)
 				goto sip404;
 			
