@@ -130,6 +130,7 @@ public:
 	const char *answer;
 	const char *reconnect;
 	const char *caller;
+	const char *hold, *transfer, *update;
 	timeout_t framing, accept;
 	int regid;
 	unsigned short call_limit;
@@ -253,7 +254,7 @@ protected:
 public:
         uint32  iBytes;
         uint32  oBytes;
-        uint32  oTimestamp;
+        uint32  oTimestamp, pTimestamp, interval;
 
 	RTPStream(Session *session);
 	~RTPStream();
@@ -304,7 +305,7 @@ protected:
 	uint8 dtmf_payload, data_payload;
 	uint8 dtmf_saved, data_saved;
 	Info info_saved;
-	bool dtmf_sipinfo;
+	bool dtmf_sipinfo, sip_answered;
 	bool reinvite, reconnecting, noinvite;
 	const char *encoding_recall;
 	timeout_t framing_recall;
@@ -320,7 +321,7 @@ protected:
 
 	void setDTMFMode(const char *mode);
 
-	void redirect(const char *target, const char *encoding, unsigned char data, unsigned char dtmf);
+	void redirect(const char *target, const char *encoding, unsigned char data, unsigned char dtmf, timeout_t framing = 0);
 	void reconnect(void);
 
 public:
@@ -363,6 +364,8 @@ public:
 
 	bool enterHangup(Event *event);
 
+	bool enterAnswer(Event *event);
+
 	bool enterPickup(Event *event);
 
 	bool enterPlay(Event *event);
