@@ -226,7 +226,7 @@ void Session::setPeering(const char *enc)
 	setConst("session.sdp_peering", enc);
 }
 
-void Session::redirect(const char *target, const char *encoding, unsigned char data, unsigned char dtmf)
+void Session::redirect(const char *target, const char *encoding, unsigned char data, unsigned char dtmf, timeout_t framing)
 {
 	osip_message_t *msg = NULL;
 	char sdp[512];
@@ -246,7 +246,11 @@ void Session::redirect(const char *target, const char *encoding, unsigned char d
 		logname, getSymbol("session.rtp_remote"), addr, port);
 
 	eXosip_lock();
-	eXosip_call_build_request(did, "INVITE", &msg);
+	if(sip_answered)
+		eXosip_call_build_request(did, "INVITE", &msg);
+	else
+		eXosip_call_build_answer(tid, 200, &msg);
+ 
 	if(!msg)
 	{
 		eXosip_unlock();
@@ -287,13 +291,30 @@ void Session::redirect(const char *target, const char *encoding, unsigned char d
 			data, cp);
 		addString(sdp, sizeof(sdp), cbuf);
 	}
-
+	if(framing)
+	{
+		snprintf(cbuf, sizeof(cbuf),
+			"a=ptime:%ld\r\n", framing);
+		addString(sdp, sizeof(sdp), cbuf);
+	}
+	
 	if(invite)
 		addString(sdp, sizeof(sdp), invite);
 	
 	osip_message_set_body(msg, sdp, strlen(sdp));
-	osip_message_set_content_type(msg, "application/sdp");
-	eXosip_call_send_request(did, msg);
+	if(sip_answered)
+	{
+	        osip_message_set_content_type(msg, "application/sdp");
+		eXosip_call_send_request(did, msg);
+	}
+	else
+	{
+		osip_message_set_require(msg, "100rel");
+                osip_message_set_header(msg, "RSeq", "1");
+                osip_message_set_body(msg, sdp, strlen(sdp));
+                osip_message_set_content_type(msg, "application/sdp");
+		eXosip_call_send_answer(tid, 200, msg);
+	}
 	eXosip_unlock();
 	reinvite = true;
 }	
@@ -305,7 +326,7 @@ void Session::reconnect(void)
 	if(!target || !reinvite)
 		return;
 
-	redirect(target, sdpEncoding(), data_payload, dtmf_payload);
+	redirect(target, sdpEncoding(), data_payload, dtmf_payload, info.framing);
 	reinvite = false;
 }
 
@@ -532,6 +553,7 @@ void Session::makeIdle(void)
 	noinvite = false;
 	sv = 0;
 
+	sip_answered = true;
 	encoding_recall = NULL;
 
 	if(offhook)
@@ -1055,6 +1077,12 @@ failure:
 				data_payload, cp);
 			addString(sdp, sizeof(sdp), cbuf);
 		}
+		if(info.framing)
+		{
+			snprintf(cbuf, sizeof(cbuf),
+				"a=ptime:%ld\r\n", info.framing);
+			addString(sdp, sizeof(sdp), cbuf);
+		}
 
 		setConst("session.sdp_encoding", rtpmap);
 		if(reg && reg->peering)
@@ -1085,6 +1113,7 @@ failure:
 		if(cid <= 0)
 			goto failure;
 		startTimer(atol(Driver::sip.getLast("invite")));
+		sip_answered = true;
 		return true;
 	case CALL_CONNECTED:
 		startRTP();
@@ -1115,6 +1144,13 @@ bool Session::enterHangup(Event *event)
 	switch(event->id)
 	{
 	case ENTER_STATE:
+		// if never answered, we are rejecting the invite...
+		if(!sip_answered)
+		{
+			eXosip_lock();
+			eXosip_call_send_answer(tid, 488, NULL);
+			eXosip_unlock();
+		}
 		cp = getSymbol("session.registry");
 		if(!cp)
 			break;
@@ -1190,6 +1226,37 @@ bool Session::enterPlay(Event *event)
 	}
 }
 
+bool Session::enterAnswer(Event *event)
+{
+	switch(event->id)
+	{
+	case ENTER_STATE:
+		if(!sip_answered)
+		{
+			reinvite = true;
+			startTimer(1000);
+			reconnect();
+		}
+		return false;
+	case TIMER_EXPIRED:
+		if(!sip_answered)
+		{
+			sip_answered = true;
+			scriptEvent("call:failed");
+			error("no-answer");
+			setRunning();
+			return true;
+		}
+		return false;
+	case CALL_ACCEPTED:
+		startTimer(50);
+		sip_answered = true;
+		return true;
+	default:
+		return false;
+	}
+}
+
 bool Session::enterPickup(Event *event)
 {
 	timeout_t accept;
@@ -1233,6 +1300,32 @@ bool Session::enterReconnect(Event *event)
 {
 	const char *sdp;
 	const char *enc;
+	
+	if(!sip_answered)
+		switch(event->id)
+		{
+		case ENTER_RECONNECT:
+			startTimer(1000);
+			reinvite = true;
+			reconnect();
+			return true;
+		case ENTER_STATE:
+			return true;
+		case CALL_ACCEPTED:
+			startTimer(50);
+			return true;
+		case DTMF_KEYDOWN:
+		case DTMF_KEYUP:
+		case LINE_DISCONNECT:
+			return true;
+		case TIMER_EXPIRED:
+			sip_answered = true;
+			reconnecting = false;
+			setRunning();
+			return true;
+		default:
+			return false;
+		}					
 
 	switch(event->id)
 	{
