@@ -42,6 +42,9 @@ BayonneService(0, 0), Keydata("/bayonne/module/webservice")
 	        load("~bayonne/webservice");
 
 	load(defkeys);
+#ifdef	WIN32
+	loadFile("C:\\Program Files\\GNU Telephony\\Bayonne Config\\module.ini", "webservice");
+#endif
 }
 
 void Service::stopService(void)
